/*
 *	Copyright (C) 2009 by Chebotarev Roman <roma@ultranet.ru>
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <pcap.h>               /* if this gives you an error try pcap/pcap.h */

#ifdef _WIN32

#include "if_ether.h"
#include "ethernet.h"

#else

#include <arpa/inet.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>   /* includes net/ethernet.h */
#include <signal.h>
#include <time.h>
#include <arpa/inet.h>

#endif

#include "dhcp.h"
#include "dhcdrop.h"



const char usage_message[] = "Usage: "
"dhcpdrop [-h] [-D] [-t]  [-y] [-r] [-b]\n"
	"\t\t[-m <count>] [-c <count>] [-n <hostname>]\n"
	"\t\t[-N <clientname>] [-p <port>] [-P <port>]\n"
	"\t\t[-T timeout] [-l <MAC-address>]\n"
	"\t\t[-C <"CHILDREN_TYPE" count (2 - 32)>] [<initial MAC address>]\n"
	"\t\t-i <interface-name|interface-index>\n";
const char keys[] = "\
\t-m count - maximal number of attempts \n"
	"\t\tto receive answer from DHCP server.\n"
"\t-c count - maximum number of receiving addresses\n"
	"\t\tfrom DHCP server (default: 255).\n"
"\t-n hostname - value of DHCP-option 'HostName'\n"
	"\t\t(default: 'DHCP-dropper').\n"
"\t-N clientname - value of DHCP-option 'Vendor-Class'\n"
	"\t\t(default: 'DHCP-dropper').\n"
"\t-l MAC-address - ethernet address of DHCP server\n"
	"\t\twhich need to ignore. May be several servers.\n"
	"\t\tNeed option '-l' for each server.\n"
"\t-i interface - defines network interface, can be\n"
	"\t\tname or index (cannot be 'any'). \n"
	"\t\tFor listing available interfaces use option -D.\n"
"\t-C count - "
#ifdef _WIN32
	"child thread"
#else
	"children"
#endif
	" number\n"
	"\t\t(default: 0, minimal value: 2, maximum: 32).\n"
	"\t\tCompatible only with flag '-f'.\n"
#ifdef _WIN32
	"\t\tWARNING: not effectively under OS Windows.\n"
#endif
"\t-p port - set client port value.\n"
"\t-P port - set server port value.\n"
"\t-T timeout - set timeout for waiting server\n"
	"\t\tresponse in seconds (default: 3).\n"
"\t-t - test mode. Send DHCPDISCOVER and wait DHCPOFFER.\n"
	"\t\tExit with code 200 if DHCPOFFER is received.\n"
	"\t\tOtherwise - exit with code 0.\n"
"\t-D - list of available network interfaces.\n"
	"\t\tFormat - index:name.\n"
"\t-y - always answer 'yes' on all questions.\n"
"\t-r - disable ethernet address randomize.\n"
"\t-f - flood mode. Generate DHCPDISCOVER flood.\n"
"\t-b - use flag 'broadcast' in DHCP message\n"
	"\t\t(default: don't use flag broadcast).\n"
"\t-a - always wait server response\n"
	"\t\ton default DHCP client port (68)\n"
"\t-A - always wait server response\n"
	"\t\tfrom default DHCP server port (67)\n"
"\t-h - print this help message\n";

const char ret_codes[] = "Program can exit with codes:\n\
\t0   - Exit success. Illegal DHCP server not found.\n\
\t10  - invalid user ID. You must be root for running programm.\n\
\t20  - failed to set signal handler.\n\
\t30  - configuration error. See usage.\n\
\t40  - memory allocation error. Insufficient memory?\n\
\t50  - error opening ethernet device.\n\
\t51  - error listing devices.\n\
\t60  - pcap filter overflow.\n\
\t70  - pcap compile error.\n\
\t80  - pcap set filter error.\n\
\t90  - error sending packet.\n\
\t100 - error getting packet.\n\
\t110 - non blocked mode set error.\n\
\t120 - invalid device.\n\
\t200 - illegal DHCP server was found.\n";

struct	ignored_mac_node * start_ignor_mac_list;
struct	ignored_mac_node * ptr_ignor_mac_list;
struct	ignored_mac_node * end_ignor_mac_list;
int		is_timeout;

int main (int argc, char **argv)
{
	set_timer_handler(timeout);
	set_console_handler(int_sign);

    /* Init random number generator */
    srandom(time(0));

    int i, j;                               /* counters */
    char filter[1024];                      /* filter for pcap_setfilter */
    char str_dropped_mac[18];
    char errbuf[PCAP_ERRBUF_SIZE];
    pcap_t * pcap_socket;
    struct full_packet_header header;       /* Ethernet + IP + UDP headrer */
    struct dhcp_packet dhcp_data;
    int opt_len;                            /* DHCP-options length */
    int snd_data_len;                       /* Full data length for sending */

    struct config_params params;            /* Parametrs from command line */
    struct dhcp_packet *dhcp_resp = 0;
    struct bpf_program fp;

    bzero(&params, sizeof(params));
    configure(argc, argv, &params);

#ifndef _WIN32
    if(geteuid())
    {
        printf("Your EUID shoud be 0 (root) to run this program. Currently your EUID is %u. Aborting.\n", geteuid());
        exit(ERR_EUID);
    }
#endif

	if(params.discover)
		interfaces_discover(0);

    bzero(str_dropped_mac, sizeof(str_dropped_mac));

	pcap_socket = pcap_open_live(params.if_name, 65535, 1, PCAP_OPEN_LIVE_TIMEOUT, errbuf);

    if(pcap_socket == NULL)
    {
        printf("Opening device error! %s\n",errbuf);
        exit(ERR_OPENDEV);
    }
	if(pcap_datalink(pcap_socket) != DLT_EN10MB)
	{
		printf("Can't work on this link layer type! Exit.\n");
		exit(ERR_INVALID_DEV);
	}
    /*Initializing destination ethernet address*/
    memset(header.eth_head.ether_dhost, 0xFF, sizeof(header.eth_head.ether_dhost));
    /*Initializing source ethernet address*/
#define E_SHOST header.eth_head.ether_shost
    if(
       sscanf(params.str_source_mac, "%02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX",
            &E_SHOST[0], &E_SHOST[1], &E_SHOST[2], &E_SHOST[3], &E_SHOST[4], &E_SHOST[5]) < 6
        )
    {
        printf("Invalid source mac address: '%s'\n", params.str_source_mac);
		usage(0);
    }
#undef E_SHOST


#ifdef __linux__
	/* Set nonblock mode for processing DHCP timeouts */
    if(pcap_setnonblock(pcap_socket, 1, errbuf) == -1)
    {
        printf("pcap_setnoblock error: '%s'\n", errbuf);
        pcap_close(pcap_socket);
        exit(ERR_SETNOBLOCK);
    }
#endif

    if(params.flood)
    {
    	pcap_close(pcap_socket);
		printf("Sending DHCP flood. Press Ctrl+C to abort...\n");
		send_flood(&params, &header, &dhcp_data);
		return EXIT_SUCCESS;
    }
    /*Start DHCP transactions loop*/
    if(!params.test_mode)
        printf("Sending DHCPDISCOVER...\n");
    for(j = 0; j < params.max_address_count; ++j)
    {
        bzero(filter, sizeof(filter));
        bzero(params.str_source_mac, sizeof(params.str_source_mac));

#define E_SHOST header.eth_head.ether_shost
        sprintf(params.str_source_mac, "%x:%x:%x:%x:%x:%x",
				E_SHOST[0], E_SHOST[1], E_SHOST[2], E_SHOST[3], E_SHOST[4], E_SHOST[5]);
#undef E_SHOST
        /* Set start value for pcap-filter */
        sprintf(filter, STR_DHCP_FILTER, params.str_source_mac,
                (params.always_wait_from_67)? DEF_SERVER_PORT : params.server_port,
                (params.always_wait_to_68)? DEF_CLIENT_PORT : params.client_port
            );

        ptr_ignor_mac_list = start_ignor_mac_list;
        /* If dropping in process - adding server ethernet addres in pcap filter */
        if(strlen(str_dropped_mac))
        {
            if(strlen(" and ether src host ") +
                strlen(str_dropped_mac) +
                strlen(filter) > sizeof(filter) - 1)
            {
                printf("Error! Filter owerflow. Exit.\n");
                pcap_close(pcap_socket);
                exit(ERR_FILTER_OWERFLOW);
            }

            strcat(filter, " and ether src host ");
            strcat(filter, str_dropped_mac);
        }
        else
        {
            /*Adding in pcap-filter all ignored servers */
            while(ptr_ignor_mac_list)
            {
                if(strlen(" and not ether src host ") +
                    strlen(ptr_ignor_mac_list -> mac_addr) +
                    strlen(filter) > sizeof(filter) - 1)
                {
                    printf("Error! Filter owerflow. Exit.\n");
                    pcap_close(pcap_socket);
                    exit(ERR_FILTER_OWERFLOW);
                }

                strcat(filter, " and not ether src host ");
                strcat(filter, ptr_ignor_mac_list -> mac_addr);
                ptr_ignor_mac_list = ptr_ignor_mac_list -> next;
            }
        }
        /*printf("Filter: %s\n", filter);*/
        /* Compiling and set pcap-filter */
        if(pcap_compile(pcap_socket, &fp, filter, 1, 0) == -1)
        {
            perror("pcap_compile");
            pcap_close(pcap_socket);
            exit(ERR_PCAP_COMPILE);
        }

        if(pcap_setfilter(pcap_socket, &fp) == -1)
        {
            perror("pcap_setfilter");
            pcap_close(pcap_socket);
            exit(ERR_PCAP_SETFILTER);
        }

        /* Create new DHCPDISCOVER packet for new DHCP transaction */
        opt_len = make_dhcp_req(&dhcp_data, header.eth_head.ether_shost, 0, 0, 0, &params);
        snd_data_len = sizeof(struct dhcp_packet) - DHCP_OPTION_LEN + opt_len;
        if(snd_data_len < BOOTP_MIN_LEN)
            snd_data_len = BOOTP_MIN_LEN;


        /* Execute DHCP transaction - snd DHCPDISCOVER -> recv DHCPOFFER -> snd DHCPREQUEST -> recv DHCPACK */
        if(
            dhcp_transaction(pcap_socket, &header, &dhcp_data, &snd_data_len, &params, str_dropped_mac,
                            filter, sizeof(filter), dhcp_resp, &fp)
          )
        {
            if(params.rand_mac_always)/* Randomize source ethernet address for next transaction */
            {
                header.eth_head.ether_shost[0] = 0;
                for(i = 1; i < 6; ++i)header.eth_head.ether_shost[i] = random();
            }
            else header.eth_head.ether_shost[5]++; /* Inc source ethernet address for next transaction */
        }else j--;
        /* Release memory from pcap-filter programm */
        pcap_freecode(&fp);
    }

    int_sign(CLEANUP);
    pcap_close(pcap_socket);
    return ILLEGAL_SERV_FOUND;
}

/* Convert network mask to CIDR notation */
uint32_t to_cidr(const uint32_t mask)
{
	int i;
	for(i = 0; i < 33; ++i)
		if(mask & (1 << i))
			break;
	return (i == 33)? 0 : 32 - i;
}

/* Convert a numeric IP address to a string */
char *iptos(const uint32_t in)
{
	static char output[IPTOSBUFFERS][3*4 + 3 + 1];
	static short which;
	uint8_t *p;

	p = (uint8_t *)&in;
	which = (which + 1 == IPTOSBUFFERS ? 0 : which + 1);
	sprintf(output[which], "%d.%d.%d.%d", p[0], p[1], p[2], p[3]);
	return output[which];
}

void ifprint(const pcap_if_t *dev)
{
  pcap_addr_t *pcap_addr;
  /* Name */
  printf("%s", dev -> name);
  /* Description */
  if (dev->description)
    printf("\n  descr: %s", dev -> description);
  /* IP addresses */
  for(pcap_addr = dev -> addresses; pcap_addr; pcap_addr = pcap_addr -> next)
  {
	if(!pcap_addr -> addr)
		continue;
	switch(pcap_addr -> addr -> sa_family)
	{
		case AF_INET:
			if (pcap_addr -> addr)
				printf("\n  iaddr: %s", iptos(((struct sockaddr_in *)pcap_addr -> addr) -> sin_addr.s_addr));
			if (pcap_addr -> netmask)
				printf("/%d ", to_cidr(ntohl(((struct sockaddr_in *)pcap_addr -> netmask) -> sin_addr.s_addr)));
			if (pcap_addr -> broadaddr)
				printf(" bcast: %s",iptos(((struct sockaddr_in *)pcap_addr -> broadaddr) -> sin_addr.s_addr));
			if (pcap_addr -> dstaddr)
				printf(" dst addr: %s",iptos(((struct sockaddr_in *)pcap_addr -> dstaddr) -> sin_addr.s_addr));
			break;
		case AF_INET6:
		break;

		default:
		break;
		}
  }
  printf("\n");
}


#ifdef	_WIN32
BOOL WINAPI int_sign(DWORD signal)
#else
int int_sign(int32_t signal)
#endif
{
    /*Release memory from ignored addresses*/
    while(start_ignor_mac_list)
    {
        ptr_ignor_mac_list = start_ignor_mac_list -> next;
        free(start_ignor_mac_list);
        start_ignor_mac_list = ptr_ignor_mac_list;
    }

    if(signal == INTERRUPT_SIGNAL)
    {
        printf("Interrupted. Quit.\n");
        exit(EXIT_SUCCESS);
    }
	return 0;
}

unsigned short rs_crc(const unsigned short *buffer, int length)
{
    unsigned long crc = 0;
    /* Calculate CRC */
    while (length > 1)
    {
        crc += *buffer++;
        length -= sizeof (unsigned short);
    }
    if (length)
        crc += *(unsigned char*) buffer;

    crc = (crc >> 16) + (crc & 0xFFFF);
    crc += (crc >> 16);

    return (unsigned short)(~crc);
}

uint16_t make_dhcp_req
    (
        struct  dhcp_packet * dhcp_data,
        const uint8_t * ether_src_addr,
        const struct in_addr * server_address,
        const struct in_addr * req_addr,
        const int xid,
        const struct config_params * config
    )
{
	uint8_t *p_dhcp_opt;
    bzero(dhcp_data, sizeof(struct dhcp_packet));
    /* Fill common options */
    dhcp_data -> op = BOOTREQUEST;
    dhcp_data -> htype = HTYPE_ETHER;
    dhcp_data -> hlen = ETHER_ADDR_LEN;
    dhcp_data -> flags = (config -> broadcast)?htons(BOOTP_BROADCAST) : 0;
    dhcp_data -> xid = (xid)? xid : rand();						/* If xid == 0 create new xid */
    memcpy(dhcp_data -> chaddr, ether_src_addr, ETHER_ADDR_LEN);

    /*Fill dhcp-options field*/
    p_dhcp_opt = dhcp_data -> options;

    /*Start DHCP options*/
    memcpy(p_dhcp_opt, magic_cookie, sizeof(magic_cookie));
    p_dhcp_opt += sizeof(magic_cookie);

    /*Create DHCP-message type*/
    *p_dhcp_opt++ = DHO_DHCP_MESSAGE_TYPE;
    *p_dhcp_opt++ = 1;

    /* Select - make DHCPREQUEST or DHCPDISCOVER */
    if(server_address && req_addr)
    {
        *p_dhcp_opt++ = DHCPREQUEST;
        *p_dhcp_opt++ = DHO_DHCP_REQUESTED_ADDRESS;
        *p_dhcp_opt++ = sizeof(req_addr -> s_addr);
        memcpy(p_dhcp_opt, &req_addr -> s_addr, sizeof(req_addr -> s_addr));
        p_dhcp_opt += sizeof(req_addr -> s_addr);
        /*Create parametr "server identifier"*/
        *p_dhcp_opt++ = DHO_DHCP_SERVER_IDENTIFIER;
        *p_dhcp_opt++ = sizeof(server_address -> s_addr);
        memcpy(p_dhcp_opt, &server_address -> s_addr, sizeof(server_address -> s_addr));
        p_dhcp_opt += sizeof(server_address -> s_addr);
    }
    else *p_dhcp_opt++ = DHCPDISCOVER;

    /*Create parametrs request list*/
    *p_dhcp_opt++ = DHO_DHCP_PARAMETER_REQUEST_LIST;
    *p_dhcp_opt++ = 3;
    *p_dhcp_opt++ = DHO_DOMAIN_NAME_SERVERS;
    *p_dhcp_opt++ = DHO_ROUTERS;
    *p_dhcp_opt++ = DHO_SUBNET_MASK;

    /*Create parametr "hostname"*/
    *p_dhcp_opt++ = DHO_HOST_NAME;
    *p_dhcp_opt++ = strlen(config -> client_hostname);
    memcpy(p_dhcp_opt, config -> client_hostname, strlen(config -> client_hostname));
    p_dhcp_opt += strlen(config -> client_hostname);

    /*Create parametr "dhcp-client name" (Vendor-Class)*/
    *p_dhcp_opt++ = DHO_VENDOR_CLASS_IDENTIFIER;
    *p_dhcp_opt++ = strlen(config -> dhcp_client);
    memcpy(p_dhcp_opt, config -> dhcp_client, strlen(config -> dhcp_client));
    p_dhcp_opt += strlen(config -> dhcp_client);

    /*Create client-ID option*/
    *p_dhcp_opt++ = DHO_DHCP_CLIENT_IDENTIFIER;
    *p_dhcp_opt++ = 1 + ETHER_ADDR_LEN;			/* Length HW-type + ETHER_ADDR_LEN == 7 */
    *p_dhcp_opt++ = HTYPE_ETHER;
    memcpy(p_dhcp_opt, ether_src_addr, ETHER_ADDR_LEN);
    p_dhcp_opt += ETHER_ADDR_LEN;

    /*End options*/
    *p_dhcp_opt++ = DHO_END;

    return  p_dhcp_opt - dhcp_data -> options;
}



int assemble_header
    (
        struct full_packet_header * header,
        const struct in_addr src_addr,
		const struct in_addr dst_addr,
        const unsigned char *data,
		const int data_len,
        const struct config_params * params
    )
{
    /*Fill ethernet header - only ethertype*/
    header -> eth_head.ether_type = htons(ETHERTYPE_IP);
    /* Fill IP header */
    bzero(&header -> ip_header, sizeof(struct iphdr));
    header -> ip_header.ihl = IP_HDR_LEN;
    header -> ip_header.version = 4;
    header -> ip_header.tos = 0x10;
    header -> ip_header.tot_len = htons(B_IP_HDR_LEN + sizeof(struct udphdr) + data_len);
    header -> ip_header.id = (uint16_t)random();
    header -> ip_header.frag_off = 0;
    header -> ip_header.ttl = DEF_TTL;
    header -> ip_header.protocol = IPPROTO_UDP;
    header -> ip_header.check = 0;
    header -> ip_header.saddr = src_addr.s_addr;
    header -> ip_header.daddr = dst_addr.s_addr;

    header -> ip_header.check = rs_crc((unsigned short*)&header -> ip_header, sizeof(struct iphdr));
    header -> udp_header.source = htons(params -> client_port);
    header -> udp_header.dest = htons(params -> server_port);
    header -> udp_header.len = htons(sizeof(struct udphdr) + data_len);
    header -> udp_header.check = 0; /*Don't use because CRC for UDP - optional parametr*/

    return 1;
}

uint16_t set_dhcp_type(const struct dhcp_packet *request, const uint16_t new_type)
{
    uint8_t *option = (uint8_t *)request + sizeof (struct dhcp_packet) - DHCP_OPTION_LEN;
    const uint8_t * opt_end = (const uint8_t *)request + sizeof(struct dhcp_packet);
    uint8_t old_type;
    if(memcmp(option, magic_cookie, sizeof(magic_cookie)))  /* Exit if magic_cookie not found - */
            return -1;
    option += sizeof(magic_cookie);							 /* Start options field */
    while((option < opt_end) && (*option != 255))
    {
		if(*option == DHO_DHCP_MESSAGE_TYPE)
        {
			old_type = *(option + 2);
            *(option + 2) = new_type;
            return old_type;
        }
        else option += *(option + 1) + 2;
    }
    return 0;
}

int get_dhcp_option(const struct dhcp_packet *request, const uint16_t packet_len,
                        const int req_option, void * option_value)
{
    /* Calculate start address for field "options" in DHCP packet */
    uint8_t *option = (uint8_t *)request + sizeof (struct dhcp_packet) - DHCP_OPTION_LEN;
    /* End options equal end packet */
    const uint8_t * opt_end = (const uint8_t *)request + packet_len;
    if(memcmp(option, magic_cookie, sizeof(magic_cookie)))/* Check "Magic cookie" in first 4 bytes options-field */
        return 0;
    option += sizeof(magic_cookie);

    while((option < opt_end) && (*option != DHO_END))
    {
        if((*(option + 1) + option) > opt_end)
        {
            printf("WARNING! Invalid value in DHCP-option length. Attempting DoS? Sleep %d second...\n", FAIL_TIMEOUT);
            sleep(FAIL_TIMEOUT);
            return -1;
        }

        if(*option == req_option)
        {
            memcpy(option_value, option + 2, *(option + 1));
            return *(option + 1);
        }
        else option += *(option + 1) + 2;
    }
    return 0;
}

int decode_dhcp(const struct dhcp_packet * packet, const uint16_t packet_len)
{
    int i;
    uint8_t dhcp_msg_type;
    switch(packet -> op)
    {
        case BOOTREQUEST:   printf("Message type BOOTREQUEST("); break;
        case BOOTREPLY:     printf("Message type BOOTREPLY ("); break;
        default:            printf("Unknown message type: %u\n", packet -> op); return 0;
    }
    if(get_dhcp_option(packet, packet_len, DHO_DHCP_MESSAGE_TYPE, (void*)&dhcp_msg_type) < 1)
    {
        printf("Error: Can't get DHCP message type.\n");
        return 0;
    }

    switch(dhcp_msg_type)
    {
        case DHCPDISCOVER:	printf("DHCPDISCOVER)\n"); break;
        case DHCPOFFER:		printf("DHCPOFFER) "); break;
        case DHCPREQUEST:	printf("DHCPREQUEST)\n"); break;
        case DHCPDECLINE:	printf("DHCPDECLINE)\n"); break;
        case DHCPACK:		printf("DHCPACK)\n"); break;
        case DHCPNAK:		printf("DHCPNAK)\n"); break;
        case DHCPRELEASE:	printf("DHCPRELEASE)\n"); break;
        case DHCPINFORM:	printf("DHCPINFORM)\n"); break;
        default:			printf("Unknown DHCP message type: %u)\n", dhcp_msg_type);
    }

    if(packet -> flags && htons(BOOTP_BROADCAST))
		printf("flags: BROADCAST ");
    printf("client ether: ");
    for(i = 0; i < 6; ++i)
            printf((i == 5) ? "%02X " :"%02X:", packet -> chaddr[i]);
    printf("You IP: %s\n", inet_ntoa(packet -> yiaddr));
    return dhcp_msg_type;
}

#ifdef	_WIN32
void CALLBACK timeout(UINT uTimerID, UINT uMsg, DWORD_PTR dwUser, DWORD_PTR dw1, DWORD_PTR dw2)
#else
void timeout(int signal)
#endif
{
	is_timeout = 1;
}

int add_str_mac_to_ignor(struct ignored_mac_node ** start_list, struct ignored_mac_node ** end_list, char * ether_addr)
{
    int hex;
    if(sscanf(ether_addr, "%x:%x:%x:%x:%x:%x", &hex, &hex, &hex, &hex, &hex, &hex) < 6)	/* Invalid format! */
		return 0;
    if(sscanf(&ether_addr[strlen(ether_addr) - 1], "%X", &hex) < 1)
        return 0;
    if(!(*start_list))/* If ignor-list empty - create start node */
    {
        *start_list = (struct ignored_mac_node *) malloc(sizeof(struct ignored_mac_node));
        *end_list = *start_list;
    }
    else /* Add new node to end list */
    {
        (*end_list) -> next = (struct ignored_mac_node *) malloc(sizeof(struct ignored_mac_node));
        (*end_list) = (*end_list) -> next;
    }

    if(!(*end_list))/* Oops! Insufficient memory? */
    {
        perror("malloc");
        exit(ERR_MEMORY);
    }

    /* Copying info in new node */
    bzero(*end_list, sizeof(struct ignored_mac_node));
    memcpy((*end_list) -> mac_addr, ether_addr, strlen(ether_addr));

    return 1;
}

int add_mac_to_ignor(struct ignored_mac_node ** start_list, struct ignored_mac_node ** end_list, struct ether_header *eptr)
{
    if(!(*start_list))/* If ignor-list empty - create start node */
    {
        *start_list = (struct ignored_mac_node *) malloc(sizeof(struct ignored_mac_node));
        *end_list = *start_list;
    }
    else /* Add new node to end list */
    {
        (*end_list) -> next = (struct ignored_mac_node *) malloc(sizeof(struct ignored_mac_node));
        (*end_list) = (*end_list) -> next;
        (*end_list) -> next = 0;
    }
    if(!(*end_list))/* Oops! Insufficient memory? */
    {
        perror("malloc");
        exit(ERR_MEMORY);
    }
    /* Copying info in new node */
    bzero(*end_list, sizeof(struct ignored_mac_node));
#define E_SHOST	eptr -> ether_shost
     sprintf((*end_list) -> mac_addr, "%x:%x:%x:%x:%x:%x",
                    E_SHOST[0], E_SHOST[1], E_SHOST[2], E_SHOST[3], E_SHOST[4], E_SHOST[5]);
#undef E_SHOST
    return 1;
}

void usage(const char help)
{
    (help)? printf("%s%s%s", usage_message, keys, ret_codes):printf("%s", usage_message);
    printf(COPYRIGHT HOMEPAGE);
    if(!help)
        printf("Use option -h for help. Exit.\n");
    exit(ERR_CONFIG);
}

inline void rand_ether_addr(char * str_mac_addr)
{
    /* Fill first byte */
    strcat(str_mac_addr, "00:");
    char * p = str_mac_addr + strlen(str_mac_addr);
    int i;
    for(i = 0; i < 5; ++i, p +=3) /* Randomize next 5 bytes */
            sprintf(p, (i < 4)? ("%02X:"):"%02X", (int)(random() & 0xFF));
    return;
}

/*
	If num == 0 - show all available interfaces and exit from programm.
	Else - return name of interface by number 'num'
*/
char * interfaces_discover(const int num)
{
	if(!num)
		printf("Available interfaces:\n");

	char errbuf[PCAP_ERRBUF_SIZE];
	pcap_if_t *dev_list_start = 0, * dev_ptr = 0;
	char * if_name = 0;
    int32_t ret = pcap_findalldevs(&dev_list_start, errbuf);

    if(ret == -1)
    {
        printf("Error: %s\n", errbuf);
        exit(ERR_LISTING_DEV);
    }

    if(!dev_list_start)
    {
        printf("Error.\n");
        exit(ERR_LISTING_DEV);
    }

	int i = 1;
    for(dev_ptr = dev_list_start; dev_ptr; dev_ptr = dev_ptr -> next)
	{
		if(num)
		{
			if(!strcmp("any", dev_ptr -> name))
				continue;
			if((i == num))
			{
				if_name = (char *) calloc(strlen(dev_ptr -> name) + 1, sizeof(char));
				if(!if_name)
				{
					printf("Can't allocate memory for interface '%s'. Quit.\n", dev_ptr -> name);
					exit(ERR_MEMORY);
				}
				strcpy(if_name, dev_ptr -> name);
				pcap_freealldevs(dev_list_start);
				return if_name;

			}
			++i;
		}
		else
		{
			if(strcmp("any", dev_ptr -> name))
			{
				printf("%d:", i++);
				ifprint(dev_ptr);
			}
		}
	}

	pcap_freealldevs(dev_list_start);

	if(num)	/* Suitable device not found */
		return 0;

	exit(ERR_CONFIG);
}

void configure(const int argc, char **argv, struct config_params * params)
{
    bzero(params, sizeof(struct config_params));
    params -> rand_mac_always = 1;    /* Randomize source ethernet address */
    params -> max_attempts = MAX_ATTEMPTS;    /* If == 0 - exit */
    params -> server_port = DEF_SERVER_PORT;
    params -> client_port = DEF_CLIENT_PORT;

    if(argc < 2)
        usage(0);
    int i;
    for(i = 1; i < argc; ++i)
    {
        /* t y r f b h m i c n N C l p P a A T D*/
        if(argv[i][0] == '-')
        {
            /*Parsing options without params*/
            if(strlen(argv[i]) > 2)
                usage(0);
            switch(argv[i][1])
            {
            case 't':
                params -> test_mode = 1;
                continue;
            case 'y':
                params -> yes = 1;
                continue;
            case 'r':
                params -> rand_mac_always = 0;
                continue;
            case 'f':
                params -> flood = 1;
                continue;
            case 'b':
                params -> broadcast = 1;
                continue;
            case 'a':
                params -> always_wait_to_68 = 1;
                continue;
            case 'A':
                params -> always_wait_from_67 = 1;
                continue;
			case 'D':
				params -> discover = 1;
				return;
            case 'h':
                usage(1);
                break;
            }
            if(i + 1 >= argc)
                            usage(0);
            if(argv[i + 1][0] == '-')
                            usage(0);

            /*Parsing options with params*/
            switch(argv[i][1])
            {
            case 'm':
                params -> max_attempts = atoi(argv[i + 1]);
                if(params -> max_attempts < 1)
                    usage(0);
                break;
            case 'i':
                params -> if_name = argv[i + 1];
				if(atoi(params -> if_name) > 0)
				{
					params -> if_name = interfaces_discover(atoi(params -> if_name));
					if(!params -> if_name)
					{
						printf("Can't find device by index: '%s', use option '-D' for interface discover. Exit.\n",
							argv[i + 1]);
						exit(ERR_LISTING_DEV);
					}
					printf("Using interface: '%s'\n", params -> if_name);
				}
                break;
            case 'c':
                params -> max_address_count = atoi(argv[i + 1]);
                if(params -> max_address_count < 1)
                    usage(0);
                break;
            case 'n':
                if(strlen(argv[i + 1]) > sizeof(params -> client_hostname) - 1)
                {
                    printf("Client hostname too long.\n");
                    usage(0);
                }
                memcpy(params -> client_hostname, argv[i + 1], strlen(argv[i + 1]));
                break;
            case 'N':
                if(strlen(argv[i + 1]) > sizeof(params -> dhcp_client) - 1)
                {
                    printf("DHCP client name too long.\n");
                    usage(0);
                }
                memcpy(params -> dhcp_client, argv[i + 1], strlen(argv[i + 1]));
                break;
            case 'C':
                params -> children_count = atoi(argv[i + 1]);
                if(params -> children_count < 2 ||
                		params -> children_count > MAX_CHILDREN)
                    usage(0);
                break;
            case 'p':
                params -> client_port = atoi(argv[i + 1]);
                if(params -> client_port < 1 || params -> client_port > MAX_PORT_VALUE)
                {
                    printf("Invalid DHCP client port value: '%s'\n", argv[i + 1]);
                    usage(0);
                }
                break;
            case 'P':
                params -> server_port = atoi(argv[i + 1]);
                if(params -> server_port < 1 || params -> server_port > MAX_PORT_VALUE)
                {
                    printf("Invalid DHCP server port value: '%s'\n", argv[i + 1]);
                    usage(0);
                }
                break;
            case 'l':
                if(strlen(argv[i + 1]) > 17)
                {
                    printf("Legal-server ethernet address too long.\n");
					usage(0);
                }
                if(!add_str_mac_to_ignor(&start_ignor_mac_list, &end_ignor_mac_list, argv[i + 1]))
                {
                    printf("Invalid legal-server ethernet address: '%s'\n", argv[i + 1]);
					usage(0);
                }
                break;
			case 'T':
				params -> timeout = atoi(argv[i + 1]);
				if(params -> timeout < 1)
				{
					printf("Invalid timeout value: '%s'\n", argv[i + 1]);
					usage(0);
				}
				break;
            default:
                    usage(0);
                break;
            }
            ++i;
        }
        else
        {
            if(strlen(argv[i]) > sizeof(params -> str_source_mac) - 1)
            {
                printf("Length source MAC address too long. Exit.\n");
				usage(0);
            }
            if(strlen(params -> str_source_mac))
                                            usage(0);               /*Ethernet address already set*/
            strcat(params -> str_source_mac, argv[i]);
        }
    }

    if(params -> flood && params -> test_mode)
    {
        printf("Error: option '-f' (flood) not compatible with option '-t' (test mode).\n");
        usage(0);
    }
    if(params -> children_count && ! params -> flood)
    {
        printf("Error: option '-C' (children count) compatible only with option '-f' (flood).\n");
        usage(0);
    }
    if(params -> max_address_count > 255 && !params -> rand_mac_always && !params -> flood)
    {
        printf("Error: Option 'max address count' can't be greatest then 255 if you use option -r\n");
        usage(0);
    }
    if(!params -> if_name)
        usage(0);
	if(!strcmp("any", params -> if_name))
	{
		printf("Can't drop DHCP server on 'any' device! Exit.\n");
		exit(ERR_CONFIG);
	}
    /* If not set - set default value */
    if(! *params -> str_source_mac)
        rand_ether_addr(params -> str_source_mac);
    if(! *params -> client_hostname)
        strcat(params -> client_hostname, DEF_HOSTNAME);
    if(! *params -> dhcp_client)
        strcat(params -> dhcp_client, DEF_DHCPCLIENT);
    if( !params -> max_address_count && !params -> flood)
        params -> max_address_count = MAX_ADDR_COUNT;
	if( !params -> timeout)
		params -> timeout = DEF_TIMEOUT;
    return;
}

int send_packet(pcap_t* descr, struct full_packet_header * header,
                const uint8_t *snd_data, const int data_len, const struct config_params * params)
{
    struct in_addr src_addr, dst_addr;
    static unsigned char pack_buf[DHCP_MTU_MAX];

    bzero(pack_buf, sizeof(pack_buf));
    bzero(&src_addr.s_addr, sizeof(src_addr.s_addr));           /*Set src IP = 0.0.0.0*/
    memset(&dst_addr.s_addr, 0xFF, sizeof(dst_addr.s_addr));    /*Set dst IP = 255.255.255.255*/

    assemble_header(header, src_addr, dst_addr, snd_data, data_len, params);

    memcpy(pack_buf, header, sizeof(struct full_packet_header));
    memcpy(pack_buf + sizeof(struct full_packet_header), snd_data, data_len);

    if(pcap_inject(descr, pack_buf, sizeof(struct full_packet_header) + data_len) == -1)
    {
        pcap_perror(descr, "pcap_inject");
        printf("Trying to use another interface. Current interface is: %s\n", params -> if_name);
        return 0;
    }
    return 1;
}

void packet_handler(u_char *out_packet, const struct pcap_pkthdr *h,
                                   const u_char *packet)
{
    memcpy(out_packet, (u_char*)packet, h -> len);
    return;
}

int get_packet(pcap_t * descr, u_char * ether_packet, const int wait_seconds)
{
    int ret = 0;
	is_timeout = 0;
	uint32_t t = timer_start(wait_seconds, timeout);
	while(1)
    {
        usleep(CAP_TIMEOUT);		/* Waiting 100 microseconds for avoid CPU overload */
        ret = pcap_dispatch(descr, 1, packet_handler, (u_char*)ether_packet);
        if(ret < 0)
        {
            perror("pcap_dispatch");
            return -1;
        }
        if(ret || is_timeout)
		{
        	timer_stop(t);
            break;
		}
    }
    return ret;
}

#ifdef _WIN32
unsigned __stdcall start_flood( void* arg )
#else
int start_flood( void* arg )
#endif
{
	flood_params_ptr par_ptr = (flood_params_ptr) arg;
    int j = 0, i;
    int opt_len;
    int snd_data_len;
	int packets_count = par_ptr -> params -> max_address_count + ((par_ptr -> params -> max_address_count)?0:1 );
    char errbuf[PCAP_ERRBUF_SIZE];
	pcap_t * pcap_socket = pcap_open_live(par_ptr -> params -> if_name, 65535, 1,
		PCAP_OPEN_LIVE_TIMEOUT, errbuf);
	if(!pcap_socket)
	{
		printf("Opening device error! %s\n",errbuf);
		exit(ERR_OPENDEV);
	}

	while( j < packets_count )
    {
        /* Create new DHCPDISCOVER packet for new DHCP transaction */
        opt_len = make_dhcp_req(par_ptr -> dhcp_data, par_ptr -> header -> eth_head.ether_shost,
        		0, 0, 0, par_ptr -> params);
        snd_data_len = sizeof(struct dhcp_packet) - DHCP_OPTION_LEN + opt_len;
        if(snd_data_len < BOOTP_MIN_LEN)
            snd_data_len = BOOTP_MIN_LEN;

        /* Sending DHCDISCOVER */
        if(!send_packet(pcap_socket, par_ptr -> header, (uint8_t*) par_ptr -> dhcp_data, snd_data_len, par_ptr -> params))
        {
            pcap_close(pcap_socket);
            return ERR_SENDPACKET;
        }
        usleep(CAP_TIMEOUT);

        if(par_ptr -> params -> rand_mac_always)  /* Randomize source ethernet address for next DHCPDISCOVER */
        {
        	par_ptr -> header -> eth_head.ether_shost[0] = 0;
            for(i = 1; i < 6; ++i)
            	par_ptr -> header -> eth_head.ether_shost[i] = random();
        }
        if(par_ptr -> params -> max_address_count)
            ++j;
    }
    pcap_close(pcap_socket);
    return 0;
}

void new_child(const struct flood_params * p, const int child_num)
{
#ifdef _WIN32
	printf("Create thread #%d.\n", child_num + 1);
	p -> flood_threads[child_num] = (HANDLE)_beginthreadex(NULL, 0, start_flood, (void*) p, 0,
		(uint32_t *) &p -> threads_ids[child_num]);

#else
	int pid;
	pid = fork();
	if(pid)
		printf("Create child #%d with pid %d.\n", child_num + 1, pid);
	else
	{   /* Generate flood from child processes */
		exit(start_flood((void*) p));
	}

#endif
}

void wait_children(const struct flood_params * p)
{
#ifdef	_WIN32
	WaitForMultipleObjects(p -> params -> children_count, (HANDLE*) p -> flood_threads, 1, INFINITE);
#else
	int32_t pid;
	while( (pid = wait(0)) != -1)
		printf("Child with pid %d finished.\n", pid);
#endif
	return;
}

int send_flood(struct config_params * params,
			    struct full_packet_header * header, struct dhcp_packet * dhcp_data)
{
	int i;
	int ret_code;
	struct flood_params par;
	par.params = params;
	par.dhcp_data = dhcp_data;
	par.header = header;
#ifdef	_WIN32
	HANDLE		threads[MAX_THREADS_COUNT]; /* Array of threads for ICMP senders */
	DWORD 		th_id[MAX_THREADS_COUNT];	/* Array of threads identifiers */
	par.flood_threads = threads;
	par.threads_ids = th_id;
#endif

	if(params -> children_count < 2)
	{
		if( (ret_code = start_flood((void*) &par) ) )
			exit(ret_code);
		return 0;
	}
#ifdef _WIN32
	printf("WARNING: multithread flood not effectively under OS Windows!\n");
#endif
	printf("Create %d "CHILDREN_TYPE" for flood.\n", params -> children_count);
	for( i = 0; i < params -> children_count; ++i)
		new_child(&par, i);
	printf("Waiting for all "CHILDREN_TYPE".\n");
	wait_children(&par);
	return 1;
}


int dhcp_transaction(pcap_t * pcap_socket,
					struct full_packet_header * header,
					struct dhcp_packet *dhcp_data,
					int * snd_data_len,
					struct config_params *params,
					char *str_dropped_mac,
					char * filter,
					int filter_len,
					struct dhcp_packet *dhcp_resp,
					struct bpf_program *fp
)
{
    int     i = 0;
    int     ret = 0;
    struct  ether_header *eptr;              /* net/ethernet.h */
    uint8_t    ether_packet[DHCP_MTU_MAX];
    static int grab_ip_count = 0;
    uint8_t dhcp_msg_type;                  /*DHCP message type: DHCPDISCOVER, DHCPOFFER, etc...*/
    uint32_t lease_time;                    /*DHCP lease time*/
    int     opt_len;
    /*Waiting dhcp-response*/
    struct  iphdr *resp_ip;
    struct  udphdr *resp_udp;
    unsigned short udp_data_len = 0;
    struct  in_addr ip;
    static char dropping = 0;
    char    yN[2];
    int     max_attempts = params -> max_attempts;
    char    recv_ack_failure = 0;

    while(1)
    {   /*Sending DHCPDISCOVER*/
        if(!send_packet(pcap_socket, header, (uint8_t*)dhcp_data, *snd_data_len, params))
        {
            exit(ERR_SENDPACKET);
        }
        /*Wating DHCPOFFER(S)*/
        ret = get_packet(pcap_socket, ether_packet, params -> timeout);
        if(ret < 0)
            exit(ERR_GETPACKET);
        if(!ret)
        {
            if(params -> test_mode && !max_attempts)
                exit(EXIT_SUCCESS);
            (--max_attempts)?:exit(EXIT_SUCCESS);
            if(!params -> test_mode)
                printf("Wait timeout. Resending DHCPDISCOVER\n");
            continue;
        }
        /* Analysing packet */
        eptr = (struct ether_header *) ether_packet;
        resp_ip = (struct iphdr *)(ether_packet + sizeof(struct ether_header));
        resp_udp = (struct udphdr *)(ether_packet + sizeof(struct ether_header) + sizeof(struct iphdr));
        dhcp_resp = (struct dhcp_packet *) (ether_packet + sizeof(struct ether_header) +
                    sizeof(struct iphdr) + sizeof(struct udphdr));
        udp_data_len = ntohs(resp_udp -> len) - sizeof(struct udphdr);	/* UDP data length equal DHCP-packet length */

        if(udp_data_len > 1472)
        {
            printf("UDP data len too long (%u bytes)! Attempting DoS? Sleep %d seconds and resending DHCPDISCOVER...\n",
                                udp_data_len, FAIL_TIMEOUT);
            sleep(FAIL_TIMEOUT);
            return 0;
        }

        if(memcmp(dhcp_resp -> chaddr, header -> eth_head.ether_shost, ETHER_ADDR_LEN))
        {
            if(!params -> test_mode)
                printf("INFO: Foreigner DHCP packet - skipping...\n");
            continue;
        }

        if(get_dhcp_option(dhcp_resp, udp_data_len, DHO_DHCP_MESSAGE_TYPE, (void*)&dhcp_msg_type) < 1)
        {
            printf("WARN: Can't get DHCP message type.\n");
            continue;
        }

        if(!dropping)
        {
            if(!(ret = get_dhcp_option(dhcp_resp, udp_data_len,
                    DHO_DHCP_SERVER_IDENTIFIER, (void*)&ip.s_addr)))
                                                    printf("Can't get server id option!\n");
            if(ret < 0)
                continue;/*Possible DoS*/
             if(params -> test_mode) /* Quiet exit */
            {
				printf("DHCP SRV: %s ", inet_ntoa(ip));
				ip.s_addr = resp_ip -> saddr;
				printf("(IP-hdr: %s) SRV ether: ", inet_ntoa(ip));
                for(i = 0; i < 6; ++i)
                            printf((i == 5) ? "%02X" :"%02X:", eptr -> ether_shost[i]);
				printf(" YIP: %s\n", inet_ntoa(dhcp_resp -> yiaddr));
                int_sign(CLEANUP);
                exit(ILLEGAL_SERV_FOUND);
            }
           /* Decoding info about server */
            printf("Got response from server %s ", inet_ntoa(ip));
            ip.s_addr = resp_ip -> saddr;
            printf("(IP-header %s), server ethernet address: ", inet_ntoa(ip));
            for(i = 0; i < 6; ++i)
                        printf((i == 5) ? "%02X, " :"%02X:", eptr -> ether_shost[i]);
            if(get_dhcp_option(dhcp_resp, udp_data_len, DHO_DHCP_LEASE_TIME, (void*)&lease_time) > 0)
                                                                  printf("lease time: %.2gh (%lus)\n",
                                                                  (float)ntohl(lease_time)/3600, ntohl(lease_time));
            /*Print DHCP-info from packet*/
            decode_dhcp(dhcp_resp, udp_data_len);
        }
        if(dhcp_msg_type == DHCPOFFER ||
            (dhcp_msg_type == DHCPACK && recv_ack_failure))/* dhcp_ack_recv))*/
        {
            /* Check various parameters */
            if(dhcp_resp -> xid != dhcp_data -> xid)
            {   /* Packet dupe, or was late? */
                printf("Invalid xid value in DHCPOFFER! Should be %u but received %u. Sending DHCPREQUEST aborting.\n",
                        dhcp_data -> xid, dhcp_resp -> xid);
                continue;
            }
            if(!dropping)
            {
                yN[0] = 'n';
                (params -> yes)? yN[0] = 'y' : (printf("Drop him? [y/n] "),scanf("%s", yN));
                if(yN[0] != 'y') /* Don't drop this server */
                {    /*Add ethernet address in ignored list*/
                    add_mac_to_ignor(&start_ignor_mac_list, &end_ignor_mac_list, eptr);
                    printf("Searching next server...\n");
                    return 0;
                }
                else
                {   /*Dropping this server*/
#define	E_SHOST	eptr -> ether_shost
                    sprintf(str_dropped_mac, "%x:%x:%x:%x:%x:%x",
                            E_SHOST[0], E_SHOST[1], E_SHOST[2], E_SHOST[3], E_SHOST[4], E_SHOST[5]);
#undef	E_SHOST

                    dropping = 1;
                    /* Adding server ethernet address in pcap-filter for received packets only from him */
                    if(strlen(" and ether src host ") +
                            strlen(str_dropped_mac) +
                            strlen(filter) > filter_len - 1)
                    {
                        printf("Error! Filter owerflow. Exit.\n");
                        exit(ERR_FILTER_OWERFLOW);
                    }
                    strcat(filter, " and ether src host ");
                    strcat(filter, str_dropped_mac);
                    /*Reinit pcap filter*/
                    pcap_freecode(fp);
                    if(pcap_compile(pcap_socket, fp, filter, 1, 0) == -1)
                    {
                        perror("pcap_compile");
                        exit(ERR_PCAP_COMPILE);
                    }
                    if(pcap_setfilter(pcap_socket, fp) == -1)
                    {
                        perror("pcap_setfilter");
                        exit(ERR_PCAP_SETFILTER);
                    }
                }
            }
            if(recv_ack_failure) /* For dummy DHCP servers */
                sleep(FAIL_TIMEOUT);
            /* Create and send DHCPREQUEST */
            bzero(&dhcp_data -> options, sizeof(dhcp_data -> options));
            if(get_dhcp_option(dhcp_resp, udp_data_len, DHO_DHCP_SERVER_IDENTIFIER, (void*)&ip.s_addr) < 1)
                                                        printf("Can't get server id option!\n");

            opt_len = make_dhcp_req(dhcp_data, header -> eth_head.ether_shost, &ip, &dhcp_resp -> yiaddr,
                                        dhcp_resp -> xid, params);

            *snd_data_len = sizeof(struct dhcp_packet) - DHCP_OPTION_LEN + opt_len;

            if(!send_packet(pcap_socket, header, (uint8_t*)dhcp_data, *snd_data_len, params))
            {
                exit(ERR_SENDPACKET);
            }

            /* Waiting DHCP-ACK */
            ret = get_packet(pcap_socket, ether_packet, params -> timeout);
            if(ret < 0)
                exit(ERR_GETPACKET);
            if(ret)
            {
                eptr = (struct ether_header *) ether_packet;
                resp_ip = (struct iphdr *)(ether_packet + sizeof(struct ether_header));
                dhcp_resp = (struct dhcp_packet *) (ether_packet + sizeof(struct ether_header) +
                            sizeof(struct iphdr) + sizeof(struct udphdr));
                if(dhcp_resp -> xid != dhcp_data -> xid)
                {   /* Packet dupe, or was late? */
                    printf("Invalid xid value in DHCPACK! Should be %u but received %u.\n",
                        dhcp_data -> xid, dhcp_resp -> xid);
                }
                else
                    if(get_dhcp_option( dhcp_resp, udp_data_len,
                            DHO_DHCP_MESSAGE_TYPE, (void*)&dhcp_msg_type) < 1)
                                                                printf("Can't get DHCP message type.\n");
                    else
                    if(dhcp_msg_type == DHCPACK)
                    {   /* DHCP transaction successfully completed. Print info and exit from loop. */
                        printf("%d. Got DHCPACK from ", ++grab_ip_count);
                        for(i = 0; i < 6; ++i)
                            printf((i == 5) ? "%02X " :"%02X:", eptr -> ether_shost[i]);
                        printf("for ether addr: ");
                        for(i = 0; i < 6; ++i)
                            printf((i == 5) ? "%02X " :"%02X:", header -> eth_head.ether_shost[i]);
                        printf(" You IP: %s\n", inet_ntoa(dhcp_resp -> yiaddr));
                        max_attempts = params -> max_attempts;
                        recv_ack_failure = 0;
                        break; /*Exit from while(1)*/
                    }
                    else
                    {
                        printf("Got not DHCPACK message from server. Message has code: %u\n", dhcp_msg_type);
                        return 0;
                    }
            }
            else
            {
                (--max_attempts)?: exit(EXIT_SUCCESS);
                printf("Wait DHCPACK timeout. Resending DHCPREQUEST\n");
                recv_ack_failure = 1;
            }
        }
        else
        {
            printf("WARN: Packet type not DHCPOFFRER or DHCPACK! Sending DHCPREQUEST aborting.\n");
            return 0;
        }
    }
    return ILLEGAL_SERV_FOUND;
}


